@@ -1,10 +1,11 @@
 # autoconf -- create `configure' using m4 macros
-# Copyright (C) 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+# Copyright (C) 2001-2004, 2006-2007, 2009-2012 Free Software
+# Foundation, Inc.
 
-# This program is free software; you can redistribute it and/or modify
+# This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2, or (at your option)
-# any later version.
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,9 +13,7 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-# 02111-1307, USA.
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package Autom4te::General;
 
@@ -33,11 +32,13 @@ used in several executables of the Autoconf and Automake packages.
 
 =cut
 
-use 5.005_03;
+use 5.006_002;
 use Exporter;
 use Autom4te::ChannelDefs;
 use Autom4te::Channels;
+use Autom4te::Getopt ();
 use File::Basename;
+use File::Path ();
 use File::stat;
 use IO::File;
 use Carp;
@@ -54,7 +55,7 @@ my @export_vars =
 # Functions we define and export.
 my @export_subs =
   qw (&debug
-      &getopt &mktmpdir
+      &getopt &shell_quote &mktmpdir
       &uniq);
 
 # Functions we forward (coming from modules we use).
@@ -95,7 +96,7 @@ $force = undef;
 
 =item C<$help>
 
-Set to the help message associated to the option C<--help>.
+Set to the help message associated with the option C<--help>.
 
 =cut
 
@@ -104,7 +105,7 @@ $help = undef;
 
 =item C<$me>
 
-The name of this application, as should be used in diagostic messages.
+The name of this application, for diagnostic messages.
 
 =cut
 
@@ -191,21 +192,8 @@ sub END
 
   if (!$debug && defined $tmp && -d $tmp)
     {
-      if (<$tmp/*>)
-	{
-	  if (! unlink <$tmp/*>)
-	    {
-	      print STDERR "$me: cannot empty $tmp: $!\n";
-	      $? = 1;
-	      return;
-	    }
-	}
-      if (! rmdir $tmp)
-	{
-	  print STDERR "$me: cannot remove $tmp: $!\n";
-	  $? = 1;
-	  return;
-	}
+      local $SIG{__WARN__} = sub { $status = 1; warn $_[0] };
+      File::Path::rmtree $tmp;
     }
 
   # This is required if the code might send any output to stdout
@@ -245,56 +233,60 @@ sub debug (@)
 
 =item C<getopt (%option)>
 
-Wrapper around C<Getopt::Long>.  In addition to the user C<option>s,
-support C<-h>/C<--help>, C<-V>/C<--version>, C<-v>/C<--verbose>,
-C<-d>/C<--debug>, C<-f>/C<--force>.  Conform to the GNU Coding
-Standards for error messages.  Try to work around a weird behavior
-from C<Getopt::Long> to preserve C<-> as an C<@ARGV> instead of
-rejecting it as a broken option.
+Wrapper around C<Autom4te::Getopt::parse_options>.  In addition to
+the user C<option>s, support C<-h>/C<--help>, C<-V>/C<--version>,
+C<-v>/C<--verbose>, C<-d>/C<--debug>, C<-f>/C<--force>.  Conform to
+the GNU Coding Standards for error messages.
 
 =cut
 
 # getopt (%OPTION)
 # ----------------
 # Handle the %OPTION, plus all the common options.
-# Work around Getopt bugs wrt `-'.
 sub getopt (%)
 {
   my (%option) = @_;
-  use Getopt::Long;
-
-  # F*k.  Getopt seems bogus and dies when given `-' with `bundling'.
-  # If fixed some day, use this: '' => sub { push @ARGV, "-" }
-  my $stdin = grep /^-$/, @ARGV;
-  @ARGV = grep !/^-$/, @ARGV;
   %option = ("h|help"     => sub { print $help; exit 0 },
 	     "V|version"  => sub { print $version; exit 0 },
 
-	     "v|verbose"    => \$verbose,
-	     "d|debug"      => \$debug,
-	     'f|force'      => \$force,
+	     "v|verbose"  => sub { ++$verbose },
+	     "d|debug"    => sub { ++$debug },
+	     'f|force'    => \$force,
 
 	     # User options last, so that they have precedence.
 	     %option);
-  Getopt::Long::Configure ("bundling", "pass_through");
-  GetOptions (%option)
-    or exit 1;
-
-  foreach (grep { /^-./ } @ARGV)
-    {
-      print STDERR "$0: unrecognized option `$_'\n";
-      print STDERR "Try `$0 --help' for more information.\n";
-      exit (1);
-    }
-
-  push @ARGV, '-'
-    if $stdin;
+  Autom4te::Getopt::parse_options (%option);
 
   setup_channel 'note', silent => !$verbose;
   setup_channel 'verb', silent => !$verbose;
 }
 
 
+=item C<shell_quote ($file_name)>
+
+Quote C<$file_name> for the shell.
+
+=cut
+
+# $FILE_NAME
+# shell_quote ($FILE_NAME)
+# ------------------------
+# If the string $S is a well-behaved file name, simply return it.
+# If it contains white space, quotes, etc., quote it, and return
+# the new string.
+sub shell_quote($)
+{
+  my ($s) = @_;
+  if ($s =~ m![^\w+/.,-]!)
+    {
+      # Convert each single quote to '\''
+      $s =~ s/\'/\'\\\'\'/g;
+      # Then single quote the string.
+      $s = "'$s'";
+    }
+  return $s;
+}
+
 =item C<mktmpdir ($signature)>
 
 Create a temporary directory which name is based on C<$signature>.
@@ -309,10 +301,11 @@ sub mktmpdir ($)
 {
   my ($signature) = @_;
   my $TMPDIR = $ENV{'TMPDIR'} || '/tmp';
+  my $quoted_tmpdir = shell_quote ($TMPDIR);
 
   # If mktemp supports dirs, use it.
   $tmp = `(umask 077 &&
-	   mktemp -d -q "$TMPDIR/${signature}XXXXXX") 2>/dev/null`;
+	   mktemp -d $quoted_tmpdir/"${signature}XXXXXX") 2>/dev/null`;
   chomp $tmp;
 
   if (!$tmp || ! -d $tmp)
